unit DbLogin;

{$mode objfpc}{$H+}

interface

uses
   Forms, StdCtrls, SysUtils, LclType, Dialogs, Buttons, ExtCtrls,
   DM_Zeos, ERDia, Classes;

type

   { TfrmErdMain }

   TfrmErdMain = class(TForm)
      btnConnect: TButton;
      cmdOpenDB: TSpeedButton;
      edDbName: TEdit;
      edSrv: TEdit;
      edPWD: TEdit;
      edUser: TEdit;
      edDLL: TEdit;
      Label1: TLabel;
      Label2: TLabel;
      Label3: TLabel;
      Label4: TLabel;
      Label5: TLabel;
      dlg: TOpenDialog;
      cmdOpenDLL: TSpeedButton;
      rgTyp: TRadioGroup;

      procedure btnConnectClick(Sender: TObject);
      procedure cmdOpenDBClick(Sender: TObject);
      procedure edDLLDblClick(Sender: TObject);
      procedure FormCloseQuery(Sender: TObject; var CanClose: Boolean);
      procedure FormCreate(Sender: TObject);
      procedure rgTypSelectionChanged(Sender: TObject);

   private
      function GetTables: AnsiString;
      function GetFields: AnsiString;
      Function GetPurRefs: AnsiString;
      Function GetPKs: AnsiString;
      Function GetPurFKs: AnsiString;

   public

   end;

var
   frmErdMain: TfrmErdMain;

implementation

{$R *.lfm}

{ TfrmErdMain }

procedure TfrmErdMain.FormCreate(Sender: TObject);
begin
   dlg.InitialDir := ExtractFilePath(Application.ExeName);
end;

procedure TfrmErdMain.FormCloseQuery(Sender: TObject; var CanClose: Boolean);
begin
   MyData.con.Connected := False;
end;

procedure TfrmErdMain.rgTypSelectionChanged(Sender: TObject);
begin
   edSrv.Enabled := rgTyp.ItemIndex = 0;
   edUser.Enabled := edSrv.Enabled;
   edPWD.Enabled := edSrv.Enabled;
   cmdOpenDB.Enabled := not edSrv.Enabled;

   case rgTyp.ItemIndex of
      0: begin
            dlg.Filter := 'MySql|libmysql*.dll';
            edDLL.Text := 'libmysql.dll';
         end;

      1: begin
            edSrv.Text := '';
            dlg.Filter := 'SqLite3|sqlite3*.dll';
            edDLL.Text := 'Sqlite3.dll';
         end;
   end;
end;

procedure TfrmErdMain.btnConnectClick(Sender: TObject);
begin
   MyData.con.Connected := False;

   Case rgTyp.ItemIndex of
      0: MyData.con.Protocol := 'MySql';
      1: MyData.con.Protocol := 'SqLite';
      else
         exit;
   end;

   MyData.con.HostName := edSrv.Text;
   MyData.con.Database := edDbName.Text;
   MyData.con.LibraryLocation := edDLL.Text;
   MyData.con.User := edUser.Text;
   MyData.con.Password := edPWD.Text;

   try	//-- Öffnen versuchen --
      MyData.con.Connected := True;
      frmERD.SetSQLs(GetTables, GetFields, GetPurRefs, GetPKs, GetPurFKs);
      frmERD.BuildERD;
      frmERD.ShowModal;

   except	//-- Fehler --
      on E: Exception do
         Application.MessageBox('Connect fehlgeschlagen!', 'DB-Connect', MB_OK);
   end;

end;

procedure TfrmErdMain.cmdOpenDBClick(Sender: TObject);
begin
   dlg.Filter := '';
   if dlg.Execute then
      edDbName.Caption := dlg.FileName;
end;

procedure TfrmErdMain.edDLLDblClick(Sender: TObject);
begin
   case rgTyp.ItemIndex of
      0: dlg.Filter := 'MySql|libmysql*.dll';
      1: dlg.Filter := 'SqLite3|sqlite3*.dll';
   end;

   if dlg.Execute then
      edDll.Caption := dlg.FileName;
end;


function TfrmErdMain.GetTables: AnsiString;
begin
   case rgTyp.ItemIndex of
      0: result := 'SELECT ucase(TABLE_NAME) as TBL' +
                ' FROM INFORMATION_SCHEMA.TABLES' +
                ' WHERE TABLE_SCHEMA = ' + quotedStr(edDbName.Text) +
                ' AND TABLE_TYPE = ' + quotedStr('BASE TABLE') +
                ' ORDER by TBL';

      1: result := 'SELECT upper(name) as TBL' +
               ' from sqlite_master' +
               ' where type = ''table'' and not (name like ''%sqlite%'')' +
               ' order by TBL';
   end;
end;

function TfrmErdMain.GetFields: AnsiString;
begin
   case rgTyp.ItemIndex of
      0: Result := 'SELECT UCASE(TABLE_NAME) AS TBL, UCASE(COLUMN_NAME) AS COL,' +
               ' DATA_TYPE as TYP, CHARACTER_MAXIMUM_LENGTH as LEN' +
               ' FROM INFORMATION_SCHEMA.COLUMNS' +
               ' WHERE TABLE_SCHEMA = ' + quotedStr(edDbName.Text) +
               ' AND UCASE(TABLE_NAME) NOT IN (' +
                  'SELECT ucase(TABLE_NAME)' +
                  ' FROM INFORMATION_SCHEMA.VIEWS' +
                  ' WHERE TABLE_SCHEMA = ' + quotedStr(edDbName.Text) + ')' +
               ' ORDER BY TBL, ORDINAL_POSITION ';

      1: Result := 'SELECT DISTINCT upper(m.name) as TBL, upper(ti.Name) as COL, ti.type as typ' +
               ' FROM sqlite_master AS m,' +
               ' pragma_table_info(m.name) as ti' +
               ' WHERE m.type = ''table'' and not (m.name like ''%sqlite%'')' +
               ' order by TBL, cid';
   end;
end;

function TfrmErdMain.GetPurRefs: AnsiString;
begin
   case rgTyp.ItemIndex of
      0: Result := 'SELECT DISTINCT UCASE(REFERENCED_TABLE_NAME) AS MST_TBL,' +
                  ' UCASE(TABLE_NAME) AS DET_TBL' +
                  ' FROM INFORMATION_SCHEMA.REFERENTIAL_CONSTRAINTS' +
                  ' WHERE CONSTRAINT_SCHEMA = ' + quotedStr(edDbName.Text) +
                  ' ORDER BY MST_TBL';

      1: Result := 'SELECT DISTINCT upper(fkl.[table]) as MST_TBL,' +
               ' upper(m.name) as DET_TBL' +
               ' FROM sqlite_master AS m,' +
               ' pragma_foreign_key_list(m.name) as fkl' +
               ' WHERE m.type = ''table''' +
               ' ORDER BY MST_TBL';
   end;
end;

function TfrmErdMain.GetPKs: AnsiString;
begin
   case rgTyp.ItemIndex of
      0: result := 'SELECT ucase(table_name) as TBL, ucase(column_name) as COL,' +
                  ' ORDINAL_POSITION as POSI' +
                  ' FROM information_schema.KEY_COLUMN_USAGE' +
                  ' WHERE CONSTRAINT_SCHEMA = ' + quotedStr(edDbName.Text) +
                  ' and CONSTRAINT_NAME = ' + quotedStr('PRIMARY') +
                  ' order by TBL, POSI';

      1: result := 'SELECT DISTINCT upper(m.name) as TBL, upper(ti.Name) as COL' +
               ' FROM sqlite_master AS m,' +
               ' pragma_table_info(m.name) as ti' +
               ' WHERE m.type = ''table'' and ti.pk > 0' +
               ' order by TBL';
   end;
end;

function TfrmErdMain.GetPurFKs: AnsiString;
begin
   case rgTyp.ItemIndex of
      0: Result := 'SELECT UCASE(TABLE_NAME) AS TBL, UCASE(COLUMN_NAME) AS COL' +
                  ' FROM INFORMATION_SCHEMA.KEY_COLUMN_USAGE' +
                  ' WHERE REFERENCED_TABLE_SCHEMA = ' + quotedStr(edDbName.Text) +
                  ' ORDER BY TBL';

      1: result := 'SELECT DISTINCT upper(m.name) as TBL,' +
                     ' upper(fkl.[from]) as COL' +
               ' FROM sqlite_master AS m,' +
               ' pragma_foreign_key_list(m.name) as fkl' +
               ' WHERE m.type = ''table''' +
               ' order by TBL';
   end;
end;

end.

