unit ERDia;

{$mode ObjFPC}{$H+}

interface

uses
   Classes, SysUtils, Forms, Controls, Graphics, Dialogs, Menus, IniFiles,
   cErdRefs, cErdTbls, DM_Zeos;


const
  csLayOut: AnsiString = '.ERDL';

type

   { TfrmERD }

   TfrmERD = class(TForm)
      dlgOpen: TOpenDialog;
      mnuSmall: TMenuItem;
      mnu: TPopupMenu;
      mnuLoadLayOut: TMenuItem;
      mnuPNG: TMenuItem;
      mnuSaveLayOut: TMenuItem;
      sBox: TScrollBox;

      procedure FormCreate(Sender: TObject);
      procedure FormDestroy(Sender: TObject);
      procedure mnuLoadLayOutClick(Sender: TObject);
      procedure mnuPNGClick(Sender: TObject);
      procedure mnuSaveLayOutClick(Sender: TObject);
      procedure mnuSmallClick(Sender: TObject);

   private
      fsLayOutDir:   AnsiSTring;
      sTitel:        AnsiString;

      oRefs:   tRefs;
      oTbls:   tRefTbls;

      slTbls:  TStringList;
      sTbls:   AnsiString;    //-- SQL für Tabellen
      sFlds:   AnsiString;    //-- SQL für Felder
      sRefs:   AnsiString;    //-- SQL für Referenzen
      sPKs:    AnsiString;    //-- SQL für PrimaryKeyFields
      sFKs:    AnsiString;    //-- SQL für ForeignKeyFields

      bToggled:   boolean;
      bIsSmall:   boolean;

      procedure TablesToSL;
      procedure CreateRefs;
      procedure ArrangeRefs;
      procedure SetAllVisi(bVisi: boolean; bWithTbls: boolean = True);
      procedure SetAllSmall;
      procedure ShowMasters(ndx: integer; var minLV: integer);
      procedure ShowDetails(ndx: integer; var maxLV: integer);
      procedure ArrangeTables(minLV, maxLV: integer);
      procedure ShowRefs;

      procedure TableMove(Sender: TObject; ID: integer);
      procedure TableDblClick(Sender: TObject; ID: integer);

   public
      procedure SetSQLs(xTbls, xFlds, xRefs, xPKs, xFKs: AnsiString);
      procedure BuildERD;
   end;

var
   frmERD: TfrmERD;

implementation

{$R *.lfm}

{ TfrmERD }

procedure TfrmERD.FormCreate(Sender: TObject);
begin
  sTitel := Caption;
  fsLayOutDir := ExtractFilePath(Application.ExeName) + 'LayOuts\';
  dlgOpen.InitialDir := fsLayOutDir;
  dlgOpen.Filter := '*' + csLayOut;
  ForceDirectories(fsLayOutDir);

  sBox.BorderSpacing.Around := 5;
  sBox.DoubleBuffered := True;
  slTbls := TStringList.Create;
  bToggled := False;
  bIsSmall := False;

  oRefs := tRefs.Create;
  oTbls := tRefTbls.Create;
end;

procedure TfrmERD.FormDestroy(Sender: TObject);
begin
  oRefs.Free;
  oTbls.Free;
  slTbls.Free;

  inherited;
end;

procedure TfrmERD.SetSQLs(xTbls, xFlds, xRefs, xPKs, xFKs: AnsiString);
begin
   sTbls := xTbls;
   sFlds := xFlds;
   sRefs := xRefs;
   sPKs := xPKs;
   sFKs := xFKs;
end;

procedure TfrmERD.BuildERD;
begin
   TablesToSL;
   oTbls.Prepare(slTbls, sBox, @TableDblClick);

   oTbls.AddFields(sFlds);
   oTbls.AddKey(sPKs, True);
   oTbls.AddKey(sFKs, False);

   CreateRefs;
end;

procedure TfrmERD.TablesToSL;
begin
   if MyData.OpenQry(sTbls) then begin
      MyData.qry.First;
      repeat
         slTbls.Append(qryFLD('TBL').AsString);
         MyData.qry.Next;
      until MyData.qry.EOF;
   end;
end;

procedure TfrmERD.CreateRefs;
   var   mst, det, ndx, c, cAdd: integer;
begin
   c := 0;

   if MyData.OpenQry(sRefs) then begin
      oRefs.Prepare(QryCount, sBox);
      cAdd := (256 * 256 * 256) div QryCount;

      repeat
         ndx := MyData.qry.RecNo -1;
         mst := slTbls.IndexOf(qryFld('MST_TBL').AsString);
         det := slTbls.IndexOf(qryFld('DET_TBL').AsString);
         oRefs.AddRef(ndx, mst, det);
         oRefs.Items[ndx].oErRef.ArrowColor := c;
         oTbls.AddMasterDetail(ndx, mst, det, @TableMove);

         inc(c, cAdd);
         MyData.qry.Next;
      until MyData.qry.EOF;

      oTbls.SetLevel;
      ArrangeRefs;
   end;
end;

procedure TfrmERD.ArrangeRefs;
   var   i, mst, det: integer;
begin
   for i := 0 to oRefs.Count -1 do begin
      mst := oRefs[i].nMstNdx;
      det := oRefs[i].nDetNdx;
      oRefs[i].oErRef.ArrowLeft := oTbls.MasterIsLeft(mst, det);
      oRefs[i].oErRef.LineUp := oTbls.LineIsUp(mst, det);
      oRefs[i].oErRef.MoveBetween(oTbls[mst].oErTbl.BoundsRect,
                                    oTbls[det].oErTbl.BoundsRect);
   end;
end;

procedure TfrmERD.TableMove(Sender: TObject; ID: integer);
   var   i, mst, det: integer;
begin
   for i := 0 to oRefs.Count -1 do begin
      mst := oRefs[i].nMstNdx;
      det := oRefs[i].nDetNdx;

      if (mst = ID) or (det = ID) then begin
         oRefs[i].oErRef.ArrowLeft := oTbls.MasterIsLeft(mst, det);
         oRefs[i].oErRef.LineUp := oTbls.LineIsUp(mst, det);
         oRefs[i].oErRef.MoveBetween(oTbls[mst].oErTbl.BoundsRect,
                                       oTbls[det].oErTbl.BoundsRect);
      end;
   end;
end;

procedure TfrmERD.mnuSmallClick(Sender: TObject);
begin
   SetAllSmall;
end;

procedure TfrmERD.TableDblClick(Sender: TObject; ID: integer);
   var   lvMin, lvMax: integer;
begin
   SetAllVisi(bToggled);
   bToggled := not bToggled;

   if bToggled then begin
      lvMax := oTbls[ID].nLev;
      lvMin := lvMax;
      oTbls[ID].oErTbl.Visible := True;
      ShowMasters(ID, lvMIN);
      ShowDetails(ID, lvMax);
      ArrangeTables(lvMin, lvMax);
      ShowRefs;
      ArrangeRefs;
      Caption := sTitel + '  [' + oTbls[ID].sTbl + ']';

   end else begin
      oTbls.ReArrange;
      ArrangeRefs;
      Caption := sTitel;
   end;
end;

procedure TfrmERD.SetAllVisi(bVisi: boolean; bWithTbls: boolean);
   var   i: integer;
begin
   if bWithTbls then
      for i := 0 to oTbls.Count - 1 do
         oTbls[i].oErTbl.Visible := bVisi;

   for i := 0 to oRefs.Count -1 do
      oRefs[i].oErRef.Visible := bVisi;
end;

procedure TfrmERD.SetAllSmall;
   var   i: integer;
begin
   bIsSmall := not bIsSmall;
   for i := 0 to oTbls.Count - 1 do
      oTbls[i].oErTbl.SetSmallSize(bIsSmall);
end;

procedure TfrmERD.ShowMasters(ndx: integer; var minLV: integer);
   var   i, ID: integer;
begin
   for i := 0 to oTbls[ndx].ilMst.Count - 1 do begin
      ID := oTbls[ndx].ilMst.Items[i];
      oTbls[ID].oErTbl.Visible := True;

      if oTbls[ID].nLev < minLV then
         minLV := oTbls[ID].nLev;

      ShowMasters(ID, minLV);
   end;
end;

procedure TfrmERD.ShowDetails(ndx: integer; var maxLV: integer);
   var   i, ID: integer;
begin
   for i := 0 to oTbls[ndx].ilDet.Count - 1 do begin
      ID := oTbls[ndx].ilDet.Items[i];
      oTbls[ID].oErTbl.Visible := True;

      if oTbls[ID].nLev > maxLV then
         maxLV := oTbls[ID].nLev;

      ShowDetails(ID, maxLV);
   end;
end;

procedure TfrmERD.ArrangeTables(minLV, maxLV: integer);
   var   i, lv, w, h, r, lLV: integer;
begin
   w := 210 + 100;
   h := 150 + 10;
   lLV := 0;

   for lv := minLV to maxLV do begin
      r := 0;
      For i := 0 To oTbls.Count -1 do begin
         if oTbls[i].nLev = lv then begin
            if oTbls[i].oErTbl.Visible then begin
               oTbls[i].oErTbl.SetPos(lLV * w, r * h);
               inc(r);
            end;
         end;
      end;

      if r > 0 then
         inc(lLV);
   end;
end;

procedure TfrmERD.ShowRefs;
   var   i, mst, det: integer;
begin
   for i := 0 to oRefs.Count -1 do begin
      mst := oRefs[i].nMstNdx;
      det := oRefs[i].nDetNdx;

      if oTbls[mst].oErTbl.Visible and oTbls[det].oErTbl.Visible then
         oRefs[i].oErRef.Visible := True;
   end;
end;

procedure TfrmERD.mnuSaveLayOutClick(Sender: TObject);
   var   i: integer;
         fiNam: AnsiString;
         ini: TMemIniFile;
begin
   fiNam := inputbox('Layout', 'Bezeichnung', '');

   if finam > '' then begin
      ini := TMemIniFile.Create(fsLayOutDir + fiNam + csLayOut);

      for i := 0 to oTbls.Count - 1 do
         if oTbls[i].oErTbl.Visible then begin
            ini.WriteInteger(oTbls[i].sTbl, 'X', oTbls[i].oErTbl.Left);
            ini.WriteInteger(oTbls[i].sTbl, 'Y', oTbls[i].oErTbl.Top);
            ini.WriteInteger(oTbls[i].sTbl, 'W', oTbls[i].oErTbl.Width);
            ini.WriteInteger(oTbls[i].sTbl, 'H', oTbls[i].oErTbl.Height);
         end;

      ini.UpdateFile;
      ini.Free;
   end;
end;

procedure TfrmERD.mnuLoadLayOutClick(Sender: TObject);
   var   i: integer;
         sl: TStringList;
         ini: TMemIniFile;
begin
   if dlgOpen.Execute then begin
      sl := TStringList.Create;
      SetAllVisi(false, False);
      ini := TMemIniFile.Create(dlgOpen.FileName);
      ini.ReadSections(sl);
      sl.Sorted := True;

      for i := 0 to oTbls.Count - 1 do
         if sl.IndexOf(oTbls[i].sTbl) > -1 then begin
            oTbls[i].oErTbl.Left := ini.ReadInteger(oTbls[i].sTbl, 'X', 0);
            oTbls[i].oErTbl.Top := ini.ReadInteger(oTbls[i].sTbl, 'Y', 0);
            oTbls[i].oErTbl.Width := ini.ReadInteger(oTbls[i].sTbl, 'W', 0);
            oTbls[i].oErTbl.Height := ini.ReadInteger(oTbls[i].sTbl, 'H', 0);
            oTbls[i].oErTbl.Visible := True;
            oTbls[i].oErTbl.Refresh;
         end else
            oTbls[i].oErTbl.Visible := False;

      ini.Free;
      sl.Free;
      ShowRefs;
      ArrangeRefs;
   end;
end;

procedure TfrmERD.mnuPNGClick(Sender: TObject);
   var   bmp: TBitmap;
         png : TPortableNetworkGraphic;
         fiNam: AnsiString;
         i: integer;
         r: TRect;
begin
   fiNam := inputbox('Layout als PNG speicher', 'Dateiname', '');
   if fiNam = '' then
      exit;

   r := oTbls.WiHi;
   bmp := TBitmap.Create;
   png := TPortableNetworkGraphic.Create;
   bmp.SetSize(r.Width +20, r.Height +20);
   bmp.Canvas.Brush.Color := clWhite;
   bmp.Canvas.Rectangle(0, 0, bmp.Width, bmp.Height);

   for i := 0 to oRefs.Count - 1 do
      if oRefs[i].oErRef.Visible then
         oRefs[i].oErRef.PaintTo(bmp.Canvas, oRefs[i].oErRef.Left +10,
                                             oRefs[i].oErRef.Top +10);

   for i := 0 to oTbls.Count - 1 do
      if oTbls[i].oErTbl.Visible then
         oTbls[i].oErTbl.PaintTo(bmp.Canvas, oTbls[i].oErTbl.Left +10,
                                             oTbls[i].oErTbl.Top +10);

   png.Assign(bmp);
   png.SaveToFile(fsLayOutDir + finam + '.png');
   bmp.Free;
   png.Free;
end;

end.

