unit ErRef;

interface

{$mode objfpc}{$H+}

uses
  Graphics, SysUtils, Classes, Controls;

const
  cArrowSize = 6;
  cShortLineLen = 20;

type
  { TErRef }

  TErRef = class(TGraphicControl)
  private    { Private declarations }
    fLineUp:      boolean;    //-- Standard: von unten nach oben
    FLineWidth:   Integer;
    FArrowColor:  TColor;

    fArrowLeft:   boolean;    //-- Master links
    aPts:         array[0..2] of TPoint;

    procedure SetLineWidth(AValue: Integer);
    procedure SetArrowColor(AValue: TColor);

  protected     { Protected declarations }
    procedure Paint; override;

  public    { Public declarations }
    constructor Create(AOwner: TComponent); override;
    destructor Destroy; override;

    procedure MoveBetween(r1, r2: TRect);
    procedure PaintTo(cnv: TCanvas; x, y: integer);

  published     { Published declarations }
    property Align;
    property Anchors;
    property Hint;
    property ShowHint;
    property Visible;

    property ArrowColor: TColor read FArrowColor write SetArrowColor default clBlack;
    property ArrowLeft: boolean read fArrowLeft write fArrowLeft default True;

    property LineUp: boolean read fLineUp write fLineUp default True;
    property LineWidth: Integer read FLineWidth write SetLineWidth default 2;

    property OnMouseDown;
    property OnMouseMove;
    property OnMouseUp;
    property OnClick;
    property OnDblClick;
  end;

implementation

uses Math;

{ TErRef }

constructor TErRef.Create(AOwner: TComponent);
begin
   inherited Create(AOwner);

   Width        := 200;
   Height       := 200;
   ShowHint     := True;

   fArrowLeft   := True;
   fLineUp      := True;
   FArrowColor  := clBlack;
   FLineWidth   := 2;
end;

destructor TErRef.Destroy;
begin
   inherited Destroy;
end;

procedure TErRef.SetLineWidth(AValue: Integer);
begin
   if AValue <> FLineWidth then begin
      FLineWidth := AValue;
      Invalidate;
   end;
end;

procedure TErRef.SetArrowColor(AValue: TColor);
begin
   if AValue <> FArrowColor then begin
      FArrowColor := AValue;
      Invalidate;
   end;
end;

procedure TErRef.MoveBetween(r1, r2: TRect);
   var   rctLeft, rctRight: TRect;
         t, l, w, h: integer;
begin
   if r1.Left < r2.Left then begin
      rctLeft := r1;
      rctRight := r2;

   end else begin
      rctLeft := r2;
      rctRight := r1;
   end;

   if rctLeft.Top < rctRight.Top then begin
      t := rctLeft.Top;
      h := rctRight.Top - t + 20;

   end else begin
      t := rctRight.Top;
      h := rctLeft.Top - t + 20;
   end;

   l := rctLeft.Left + rctLeft.Width;
   w := rctRight.Left - l;
   SetBounds(l, t, w, h);
end;

procedure TErRef.Paint;
begin
   paintTo(canvas, 0, 0);
end;

procedure TErRef.PaintTo(cnv: TCanvas; x, y: integer);
   var   ptLeft, ptLeft2, ptRight, ptRight2: TPoint;
         yDif: integer;
begin
   //-- Punkte für Z-Linie Festlegen --
   ptLeft.X := 0;
   ptRight.X := Width - 1;
   yDif := 10;

   if FLineUp then begin
      ptLeft.Y := Height - yDif;
      ptRight.Y := yDif +1;
   end else begin
      ptLeft.Y := yDif +1;
      ptRight.Y := Height - yDif;
   end;

   ptLeft.X := ptLeft.X + x;
   ptLeft.Y := ptLeft.Y + y;
   ptRight.X := ptRight.X + x;
   ptRight.Y := ptRight.Y + y;

   ptLeft2 := Point(ptLeft.X + cShortLineLen, ptLeft.Y);
   ptRight2 := Point(ptRight.X - cShortLineLen, ptRight.Y);

   cnv.Pen.Color   := FArrowColor;
   cnv.Brush.Color := FArrowColor;
   cnv.Pen.Width   := FLineWidth;

   cnv.Line(ptLeft, ptLeft2);
   cnv.Line(ptLeft2, ptRight2);
   cnv.Line(ptRight2, ptRight);

   if fArrowLeft then begin
      aPts[0] := ptLeft;
      aPts[1] := Point(cArrowSize + x, ptLeft.Y - 7);
      aPts[2] := Point(cArrowSize + x, ptLeft.Y + 7);

   end else begin
      aPts[0] := ptRight;
      aPts[1] := Point(Width + x - cArrowSize, ptRight.Y - 7);
      aPts[2] := Point(Width + x - cArrowSize, ptRight.Y + 7);
   end;

   cnv.Pen.Width := 1;
   cnv.Polygon([aPts[0], aPts[1], aPts[2]]);
end;


end.
