unit ErTbl;

{$mode objfpc}{$H+}

interface

uses
  Windows, LCLIntf, Classes, LCLType, Forms, SysUtils, Controls, ExtCtrls,
  Graphics, Grids, LResources;

const
  cErTblWidth: integer = 210;
  cErTblHeight: integer = 150;

type
  TMoveSizeMode = (msNone, msMove, msSize);

  TOnMove = procedure(Sender: TObject; ID: integer) of object;
  TOnDblClick = procedure(Sender: TObject; ID: integer) of object;

  { tErTbl }

  tErTbl = class(TCustomPanel)
  private
    fBrdrColor:   TColor;
    fBrdrSize:    integer;
    FBgColor:     TColor;
    FCaptionColor:TColor;
    FCptBgColor:  TColor;

    fFullRect:    TRect;
    FTxtRect:     TRect;
    FTextStyle:   TTextStyle;

    fMoveSize:    TMoveSizeMode;
    FLastPos:     TPoint;
    fLastHeight:  integer;

    fInitDone:    boolean;
    fGrid:        TStringGrid;

    fIndex:       integer;

    FOnMove:      TOnMove;
    FOnDblClick:  TOnDblClick;

    function  GetCells(ACol, ARow: Integer): string;
    procedure SetBrdrColor(AValue: TColor);
    procedure SetBgColor(const AValue: TColor);

    procedure PrepareRects;
    procedure DoColorBg;
    procedure DrawRect(const rct: TRect);
    procedure DrawSizer;
    procedure MyRedraw;
    procedure CreateGrid;
    procedure SetCells(ACol, ARow: Integer; AValue: string);

  protected
    procedure MouseDown(Button: TMouseButton; Shift:TShiftState; X,Y:Integer); override;
    procedure MouseMove(Shift: TShiftState; X,Y: Integer); override;
    procedure MouseUp(Button: TMouseButton; Shift:TShiftState; X,Y:Integer); override;
    procedure MouseLeave; override;

    procedure Paint; override;

    procedure DblClick; override;

  public
    constructor Create(AOwner: TComponent); override;

    Procedure SetPos(x, y: integer);
    Procedure SetSmallSize(bSetIt: boolean);
    Procedure SetHeight;
    procedure AddFld(r: integer; sFld, sTyp: AnsiString);
    procedure AddPK(sFld: AnsiString);
    procedure AddFK(sFld: AnsiString);
    procedure Refresh;

    property Anchors;
    property Caption;
    property Constraints;
    property Font;
    property Index: integer read fIndex write fIndex;
    property Visible;

    property BgColor: TColor read FBgColor write SetBgColor default clSilver;
    property BorderColor: TColor read fBrdrColor write SetBrdrColor default clGray;
    property CaptionColor: TColor read FCaptionColor write FCaptionColor default clBlack;

    property Grid: TStringGrid read fGrid;

    property OnMove: TOnMove read FOnMove write FOnMove;
    property OnDblClick: TOnDblClick read FOnDblClick write FOnDblClick;
  end;


implementation

constructor tErTbl.Create(AOwner: TComponent);
begin
   inherited create(AOwner);

   ShowHint := True;

   FBgColor       := clSilver;
   fBrdrColor     := clGray;
   fBrdrSize      := 2;
   FCaptionColor  := clBlack;
   FCptBgColor    := TColor($B0B0B0);
   Color          := FBgColor;

   Width       := cErTblWidth;
   Height      := cErTblHeight;
   Constraints.MinWidth := Width;
   fMoveSize   := msNone;
   fInitDone   := False;

   CreateGrid;

   Font.Style := [fsBold];
   FTextStyle.Alignment := taCenter;
   FTextStyle.Layout := tlCenter;
end;

procedure tErTbl.PrepareRects;
   var fntHi: Integer;
   const xD: integer = 2;
begin
   fFullRect := GetClientRect;
   FTxtRect := fFullRect;
   fntHi := Canvas.TextHeight('Tg');

   //-- Text-Rahmen bestimmen --
   Inc(FTxtRect.Top, fBrdrSize);
   Inc(FTxtRect.Left, fBrdrSize);
   Dec(FTxtRect.Right, fBrdrSize);
   FTxtRect.Bottom := fBrdrSize + fntHi + 2*xD;

   fInitDone := True;
end;

procedure tErTbl.MyRedraw;
begin
   PrepareRects;
   Paint;
end;

procedure tErTbl.CreateGrid;
   var   i: integer;
begin
   fGrid := TStringGrid.Create(self);
   fGrid.Parent := Self;
   fGrid.ParentFont := False;
   fGrid.FixedCols := 0;
   fGrid.FixedRows := 1;
   fGrid.RowCount := 50;
   fGrid.SetBounds(10, 35, cErTblWidth -20, 105);
   fGrid.Anchors := [akTop,akLeft,akRight,akBottom];
   fGrid.DefaultRowHeight := 20;

   for i := 0 to 2 do begin
      TGridColumn.Create(fGrid.Columns);
      fGrid.Columns[i].SizePriority := 0;
      fGrid.Columns[i].Title.Color := TColor($D0D0D0);
      fGrid.Columns[i].Title.Font.Style :=  [fsBold];
   end;

   fGrid.Columns[0].Width := 30;
   fGrid.Columns[1].Width := 90;
   fGrid.Columns[2].Width := 75;

   fGrid.Columns[0].Alignment := taCenter;
   fGrid.Columns[0].Font.Style := [fsBold];
   fGrid.AutoFillColumns := True;

   fGrid.Columns[0].Title.Caption := 'Key';
   fGrid.Columns[0].Title.Alignment := taCenter;
   fGrid.Columns[1].Title.Caption := 'Feld';
   fGrid.Columns[2].Title.Caption := 'Typ';
   fGrid.Visible := True;
end;

procedure tErTbl.SetPos(x, y: integer);
begin
   SetBounds(x, y, Width, Height);
end;

procedure tErTbl.SetSmallSize(bSetIt: boolean);
begin
   if bSetIt then begin
      fLastHeight := Height;
      Height := Constraints.MinHeight;

   end else
      Height := fLastHeight;
end;

procedure tErTbl.SetHeight;
   var   d: integer;
begin
   if fGrid.RowCount < 5 then begin
      d := (5 - fGrid.RowCount) * fGrid.DefaultRowHeight;
      Constraints.MaxHeight := Height - d;

   end else begin
      d := (fGrid.RowCount - 5) * fGrid.DefaultRowHeight;
      Constraints.MaxHeight := Height + d;
   end;
end;

procedure tErTbl.AddFld(r: integer; sFld, sTyp: AnsiString);
begin
   fGrid.Cells[1, r] := sFld;
   fGrid.Cells[2, r] := sTyp;
end;

procedure tErTbl.AddPK(sFld: AnsiString);
   var   i: integer;
begin
   for i := 1 to fGrid.RowCount -1 do
      if fGrid.Cells[1, i] = sFld then begin
         fGrid.Cells[0, i] := 'P';
         break;
      end;
end;

procedure tErTbl.AddFK(sFld: AnsiString);
   var   i: integer;
begin
   for i := 1 to fGrid.RowCount -1 do
      if fGrid.Cells[1, i] = sFld then begin
         if fGrid.Cells[0, i] = '' then
            fGrid.Cells[0, i] := 'F'
         else
            fGrid.Cells[0, i] := fGrid.Cells[0, i] + ' F';

         break;
      end;
end;

procedure tErTbl.Refresh;
begin
   MyRedraw;
end;

procedure tErTbl.DoColorBg;
begin
   Canvas.Brush.Color := Color;
   Canvas.FillRect(fFullRect);
   DrawRect(fFullRect);
   Canvas.Line(0, FTxtRect.Bottom, Width -1, FTxtRect.Bottom);
   Canvas.Line(0, FTxtRect.Bottom +1, Width -1, FTxtRect.Bottom +1);
   Canvas.Brush.Color := FCptBgColor;
   Canvas.FillRect(FTxtRect);
end;

procedure tErTbl.DrawRect(const rct: TRect);
   var i: integer;
begin
   Canvas.Pen.Color := fBrdrColor;
   Canvas.Pen.Width := 1;
   Canvas.Brush.Style := bsClear;

   for i := 0 to fBrdrSize - 1 do
      Canvas.Rectangle(rct.Left +i, rct.Top +i, rct.Right -i, rct.Bottom -i);
end;

procedure tErTbl.DrawSizer;
begin
   Canvas.Pen.Width := 2;
   Canvas.Pen.Color := fBrdrColor;
   Canvas.Line(Width - fBrdrSize*4, Height,
                        Width, Height - fBrdrSize*4);
   Canvas.Line(Width - fBrdrSize*7, Height,
                        Width, Height - fBrdrSize*7);
end;

procedure tErTbl.Paint;
begin
   if not fInitDone then begin
      PrepareRects;
      Constraints.MinHeight := FTxtRect.Bottom + fBrdrSize;
   end;

   DoColorBG;
   DrawSizer;

   canvas.Brush.Style := bsClear;
   canvas.Font := Font;
   canvas.Font.Color := FCaptionColor;
   canvas.TextRect(FTxtRect, FTxtRect.Left, 0, Caption, FTextStyle)
end;

procedure tErTbl.DblClick;
begin
   inherited DblClick;

   fMoveSize := msNone;

   if Assigned(FOnDblClick) then
      FOnDblClick(self, fIndex);
end;

procedure tErTbl.SetBrdrColor(AValue: TColor);
begin
   if fBrdrColor <> AValue then begin
      fBrdrColor := AValue;
      MyRedraw;
   end;
end;

function tErTbl.GetCells(ACol, ARow: Integer): string;
begin
   Result := fGrid.Cells[aCol, aRow];
end;

procedure tErTbl.SetCells(ACol, ARow: Integer; AValue: string);
begin
   fGrid.Cells[aCol, aRow] := aValue
end;

//procedure tErTbl.WMMove(var Message: TLMMove);
//begin
//   inherited WMMove(Message);
//   //
//end;

procedure tErTbl.SetBgColor(const AValue: TColor);
begin
   if FBgColor <> AValue then begin
      FBgColor := AValue;
      MyRedraw;
   end;
end;

function IsMouseOverControl(const ctrl: TControl): boolean;
   var   sr : TRect;
begin
   sr.TopLeft := ctrl.ClientToScreen(Point(0, 0));
   sr.BottomRight := ctrl.ClientToScreen(Point(ctrl.Width, ctrl.Height));
   result := PtInRect(sr, Mouse.CursorPos);
end;

procedure tErTbl.MouseLeave;
begin
   if (not enabled) or (csDesigning in ComponentState) then
      exit;

   inherited MouseLeave;

   if fMoveSize = msSize then begin
      Screen.Cursor := crDefault;
      MyRedraw;
   end;

   fMoveSize := msNone;
end;

procedure tErTbl.MouseDown(Button: TMouseButton; Shift: TShiftState;
                                       X, Y: Integer);
   const cArea: integer = 10;
begin
   if not (fMoveSize = msSize) then
      inherited MouseDown(Button, Shift, X, Y);

   if (Button = mbLeft) then begin
      FLastPos := Point(X,Y);

      if Y < FTxtRect.Bottom then
         fMoveSize := msMove
      else
         if ((Width - x ) < cArea) and ((Height - y ) < cArea) then begin
            fMoveSize := msSize;
            Screen.cursor := crSizeNWSE;
         end;
   end;
end;

procedure tErTbl.MouseMove(Shift: TShiftState; X, Y: Integer);
   var   r: TRect;
begin
   if not (fMoveSize = msSize) then
      inherited MouseMove(Shift, X, Y);

   if fMoveSize = msMove then begin
      Left := Left - FLastPos.X + x;
      Top := Top - FLastPos.Y + y;

      if Assigned(FOnMove) then
         FOnMove(self, fIndex);
   end;

   if fMoveSize = msSize then begin
      r := BoundsRect;
      SetBounds( r.left, r.top, r.right - r.left + X - FlastPos.X,
                  r.bottom - r.top + Y - Flastpos.Y );
      FLastPos := Point( x, y );
      MyRedraw;
   end;
end;

procedure tErTbl.MouseUp(Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
begin
   if not (fMoveSize = msSize) then
      inherited MouseUp(Button, Shift, X, Y);

   if fMoveSize = msSize then begin
      Screen.Cursor := crDefault;
      MyRedraw;
   end;

   fMoveSize := msNone;
end;


end.
