unit MyIntList;

{$mode objfpc}{$H+}
{$inline on}

interface

uses
  Classes, SysUtils;

const
  soInt: integer = sizeof(Int64);

type
  TInt64Array = array of Int64;

  { TIntList }
  TIntList = class(TObject)
  private
    fCapacity:    integer;
    FInts:        TInt64Array;
    FCount:       Integer;
    FGrow:        integer;
    FSorted:      boolean;
    fNoDups:      boolean;

    function  BinSearch(const Srch: Int64; var ndx: integer): boolean;
    procedure SetCapacity(AValue: integer);
    function  GetInt(Index: Integer): Int64;
    procedure PutInt(Index: Integer; const Value: Int64);
    procedure QuickSort(var IntArray: TInt64Array; Start, Stop: Integer);
    procedure SetCount(AValue: Integer);
    procedure SetDups(AValue: boolean);
    procedure SetSorted(AValue: boolean);

  public
    constructor Create(nCapa: integer = 1024);
    destructor Destroy; override;

    function  LifoPop: integer;
    function  FifoPop: integer;

    procedure Add(const I: Int64);
    procedure Clear;
    procedure Copy(const lst: TIntList);
    procedure Delete(Index: Integer);
    function  Exists(Value: Int64): boolean; overload;
    function  Exists(Value: Int64; var ndx: integer): boolean; overload;
    function  IndexOf(Value: Int64): Integer; inline;
    procedure Sort;

    property Capacity: integer read fCapacity write SetCapacity;
    property Count: Integer read FCount write SetCount;
    property Items[Index: Integer]: Int64 read GetInt write PutInt; default;
    property Sorted: boolean read FSorted write SetSorted;
    property NoDuplicates: boolean read fNoDups write SetDups;
  end;

implementation


{ TIntList }
constructor TIntList.Create(nCapa: integer);
begin
   fCount := 0;
   fCapacity := 0;
   FSorted := False;
   FGrow := nCapa shr 2;
   SetCapacity(nCapa);
end;

destructor TIntList.Destroy;
begin
   SetLength(FInts, 0);

   inherited;
end;

function TIntList.LifoPop: integer;
begin
   Result := FInts[fCount - 1];
   Delete(fCount - 1);
end;

function TIntList.FifoPop: integer;
begin
   Result := FInts[0];
   Delete(0);
end;

procedure TIntList.SetCapacity(AValue: integer);
begin
   if fCapacity <> AValue then begin
      fCapacity := AValue;
      SetLength(FInts, fCapacity);
   end;
end;

procedure TIntList.Add(const I: Int64);
begin
   if fNoDups then
      if Exists(I) then
         exit;

   //-- geht noch was ?
   if fCount = fCapacity then
      SetCapacity(fCapacity + FGrow);

   FInts[fCount] := I;
   inc(fCount);

   if fSorted then
      QuickSort(FInts, 0, FCount - 1);
end;

procedure TIntList.SetCount(AValue: Integer);
begin
   if AValue > fCapacity then
      SetCapacity(AValue);

   FCount := AValue;
   FillByte(FInts[0], Length(FInts) * soInt, 0);
end;

procedure TIntList.SetDups(AValue: boolean);
begin
   if fNoDups <> AValue then begin
      fNoDups := AValue;
      if fNoDups then
         Sorted := True;
   end;
end;

procedure TIntList.Clear;
begin
   fCount := 0;
end;

procedure TIntList.Copy(const lst: TIntList);
begin
   fCount := lst.Count;

   if fCount > 0 then
      Move(lst.FInts[0], FInts[0], fCount * soInt);
end;

procedure TIntList.Delete(Index: Integer);
begin
   Dec(fCount);
   if Index < fCount then
      Move(FInts[Index + 1], FInts[Index], (fCount - Index) * soInt);
end;

function TIntList.GetInt(Index: Integer): Int64;
begin
   Result := FInts[Index];
end;

procedure TIntList.PutInt(Index: Integer; const Value: Int64);
begin
   FInts[Index] := Value;
end;

function TIntList.Exists(Value: Int64): boolean;
begin
   if fCount > 0 then
      Result := IndexOf(Value) > -1
   else
      Result := False;
end;

function TIntList.Exists(Value: Int64; var ndx: integer): boolean;
begin
   if fCount > 0 then begin
      ndx :=  IndexOf(Value);
      Result := ndx > -1;

   end else
      Result := False;
end;

function TIntList.IndexOf(Value: Int64): Integer;
   var   ndx: integer;
begin
   if BinSearch(Value, ndx) then
      Result := ndx
   else
      Result := -1;
end;

procedure TIntList.SetSorted(AValue: boolean);
begin
   if FSorted <> AValue then
      FSorted := AValue;

   if FSorted then
      if FCount > 1 then
         QuickSort(FInts, 0, FCount - 1);
end;

procedure TIntList.Sort;
begin
   if FCount > 1 then
      QuickSort(FInts, 0, FCount - 1);
   FSorted := True;
end;

procedure TIntList.QuickSort(var IntArray: TInt64Array; Start, Stop: Integer);
   var   Left, Right, Mid: UInt32;
         Pivot, Temp: Int64;
begin
   Left  := Start;
   Right := Stop;
   Mid   := (Start + Stop) shr 1;

   Pivot := IntArray[mid];
   repeat
      while Pivot > IntArray[Left] do
         Inc(Left);
      while Pivot < IntArray[Right] do
         Dec(Right);

      if Left <= Right then begin
         Temp := IntArray[Left];
         IntArray[Left] := IntArray[Right];
         IntArray[Right] := Temp;
         Inc(Left);
         Dec(Right);
      end;
   until Left > Right;

   if Start < Right then
      QuickSort(IntArray, Start, Right);
   if Left < Stop then
      QuickSort(IntArray, Left, Stop);
end;

function TIntList.BinSearch(const Srch: Int64; var ndx: integer): boolean;
   var   First, Last, Pivot: UInt32;
begin
   First := 0;
   Last := fCount - 1;
   Result := False;

   while (First <= Last) and (not Result) do begin
      Pivot := (First + Last) shr 1;
      if FInts[Pivot] = Srch then begin
         ndx := Pivot;
         Result := True;

      end else
         if FInts[Pivot] > Srch then
            Last := Pivot - 1
         else
            First := Pivot + 1;
   end;
end;


end.
