unit cErdTbls;

{$mode objfpc}{$H+}

interface

uses
   Classes, SysUtils, Forms,
   DM_Zeos, MyIntList, ErTbl;

type
  tRefTbl = record
      sTbl:       AnsiString; //-- TableName

      nLev:       integer;
      ilMst:      TIntList;   //-- Indices der Masters
      ilDet:      TIntList;   //-- dto. Details

      oErTbl:  tErTbl;
      bIsVisi: boolean;
      bHiLi:   boolean;    // Highlighted
  end;

  { tRefTbls }
  tRefTbls = class(TObject)
    private
      aTbls:      array of tRefTbl;
      nCount:     integer;
      nMaxLev:    integer;

      procedure Clear;
      function FindTbl(sTbl: AnsiString): integer;
      function Get(Index: Integer): tRefTbl;

    public
   	constructor Create;
      destructor Destroy; override;

      procedure Prepare(slTbls: TSTringlist; sBox: TScrollBox; DoDblClick: TOnDblClick);
      procedure AddFields(sQryFlds: AnsiString);
      procedure AddKey(sQryKey: AnsiString; bPrimary: boolean);
      procedure AddMasterDetail(RefNdx, MstNdx, DetNdx: integer; DoMove: TOnMove);
      procedure SetLevel;
      procedure ReArrange;
      function  WiHi: TRect;

      Function  HasDetail(ndx: integer): boolean;
      Function  MasterIsLeft(mst, det: integer): boolean;
      Function  LineIsUp(mst, det: integer): boolean;

      property  Count: integer read nCount;
      property  Maxlevel: integer read nMaxLev;
      property  Items[Index: Integer]: tRefTbl read Get; default;
  end;


implementation

//##############################################################################
{---- tRefTbls ----}
//##############################################################################

constructor tRefTbls.Create;
begin
   inherited;

   nCount := 0;
end;

destructor tRefTbls.Destroy;
begin
   Clear;
   SetLength(aTbls, 0);

   inherited;
end;

procedure tRefTbls.Prepare(slTbls: TSTringlist; sBox: TScrollBox;
                                                      DoDblClick: TOnDblClick);
   var   i: integer;
begin
   if nCount > 0 then
      Clear;

   nCount := slTbls.Count;
   SetLength(aTbls, nCount);

   for i := 0 to nCount -1 do begin
      aTbls[i].sTbl := slTbls.Strings[i];
      aTbls[i].ilMst := TIntList.Create(16);
      aTbls[i].ilDet := TIntList.Create(16);
      aTbls[i].bIsVisi := True;
      aTbls[i].nLev := 0;

      aTbls[i].oErTbl :=  tErTbl.Create(sBox);
      aTbls[i].oErTbl.Index := i;
      aTbls[i].oErTbl.Parent := sBox;
      aTbls[i].oErTbl.Caption := aTbls[i].sTbl;
      aTbls[i].oErTbl.OnDblClick := DoDblClick;
   end;
end;

procedure tRefTbls.AddFields(sQryFlds: AnsiString);
   var   ndx, r: integer;
begin
   if not MyData.OpenQry(sQryFlds) then
      exit;

   ndx := 0;
   MyData.qry.First;

   repeat
      r := 0;
      while not MyData.qry.EOF and
                        (aTbls[ndx].sTbl = QryFld('TBL').AsString) do begin
         inc(r);
         aTbls[ndx].oErTbl.AddFld(r, QryFld('COL').AsString,
                                       QryFld('TYP').AsString);
         MyData.qry.Next;
      end;

      aTbls[ndx].oErTbl.Grid.RowCount := r +1;
      aTbls[ndx].oErTbl.SetHeight;
      inc(ndx);
   until MyData.qry.EOF;
end;

procedure tRefTbls.AddKey(sQryKey: AnsiString; bPrimary: boolean);
   var   ndx: integer;
         tbl: AnsiString;
begin
   if not MyData.OpenQry(sQryKey) then
      exit;

   tbl := '';
   MyData.qry.First;

   repeat
      if QryFld('TBL').AsString <> tbl then begin
         tbl := QryFld('TBL').AsString;
         ndx := FindTbl(tbl);
      end;

      if bPrimary then
         aTbls[ndx].oErTbl.AddPK(QryFld('COL').AsString)
      else
        aTbls[ndx].oErTbl.AddFK(QryFld('COL').AsString);

      MyData.qry.Next;
   until MyData.qry.EOF;
end;

procedure tRefTbls.AddMasterDetail(RefNdx, MstNdx, DetNdx: integer; DoMove: TOnMove);
begin
   aTbls[DetNdx].ilMst.Add(MstNdx);
   aTbls[DetNdx].oErTbl.OnMove := DoMove;

   aTbls[MstNdx].ilDet.Add(DetNdx);
   aTbls[MstNdx].oErTbl.OnMove := DoMove;
end;

procedure tRefTbls.SetLevel;
   var   i, k, det: integer;
         chg: boolean;
begin
   repeat
      chg := False;

      For i := 0 To nCount -1 do begin
         if aTbls[i].ilDet.Count > 0 then begin
            For k := 0 To aTbls[i].ilDet.Count -1 do begin
               det := aTbls[i].ilDet.Items[k];

               If aTbls[det].nLev <= aTbls[i].nLev Then begin
                  aTbls[det].nLev := aTbls[i].nLev + 1;
                  chg := True;
               End;
            End;
         end;
      end;
   Until (chg = False);

   nMaxLev := 0;
   For i := 0 To nCount -1 do begin
      if (aTbls[i].ilMst.Count = 0) and (aTbls[i].ilDet.Count = 0) then
         aTbls[i].nLev := -1;
      if aTbls[i].nLev > nMaxLev then
         nMaxLev := aTbls[i].nLev;
   end;

   ReArrange;
end;

procedure tRefTbls.ReArrange;
   var   i, lv, w, h, r: integer;
begin
   w := cErTblWidth + 100;
   h := cErTblHeight + 10;

   for lv := -1 to nMaxLev do begin
      r := 0;
      For i := 0 To nCount -1 do begin
         if aTbls[i].nLev = lv then begin
            aTbls[i].oErTbl.Visible := True;
            aTbls[i].oErTbl.SetPos((lv +1) * w, r * h);
            inc(r);
         end;
      end;
   end;
end;

function tRefTbls.WiHi: TRect;
   var   i: integer;
begin
   Result := Rect(0, 0, 0, 0);
   for i := 0 to nCount -1 do begin
      if aTbls[i].oErTbl.Visible then begin
        if aTbls[i].oErTbl.Top + aTbls[i].oErTbl.Height > Result.Height then
           Result.Height := aTbls[i].oErTbl.Top + aTbls[i].oErTbl.Height;
        if aTbls[i].oErTbl.Left + aTbls[i].oErTbl.Width > Result.Width then
           Result.Width := aTbls[i].oErTbl.Left + aTbls[i].oErTbl.Width;
      end;
   end;
end;

function tRefTbls.HasDetail(ndx: integer): boolean;
begin
   Result := aTbls[ndx].ilDet.Count > 0;
end;

function tRefTbls.MasterIsLeft(mst, det: integer): boolean;
begin
   Result := aTbls[mst].oErTbl.Left < aTbls[det].oErTbl.Left;
end;

function tRefTbls.LineIsUp(mst, det: integer): boolean;
   var   l, r: integer;
begin
   if aTbls[mst].oErTbl.Left < aTbls[det].oErTbl.Left then begin
      l := mst;
      r := det;

   end else begin
      l := det;
      r := mst;
   end;

   Result := aTbls[l].oErTbl.Top > aTbls[r].oErTbl.Top;
end;

procedure tRefTbls.Clear;
   var   i: integer;
begin
   for i := 0 to nCount -1 do begin
      aTbls[i].ilMst.Free;
      aTbls[i].ilDet.Free;
      aTbls[i].oErTbl.Free;
   end;
end;

function tRefTbls.FindTbl(sTbl: AnsiString): integer;
   var   i: integer;
begin
   for i := 0 to nCount -1 do
      if aTbls[i].sTbl = sTbl then begin
         Result := i;
         break;
      end;
end;

function tRefTbls.Get(Index: Integer): tRefTbl;
begin
   Result := aTbls[Index];
end;


end.

